@@ -10,15 +10,15 @@ _tokenizer = _Tokenizer()
 
 def weights_init_kaiming(m):
     classname = m.__class__.__name__
-    if classname.find('Linear') != -1:
-        nn.init.kaiming_normal_(m.weight, a=0, mode='fan_out')
+    if classname.find("Linear") != -1:
+        nn.init.kaiming_normal_(m.weight, a=0, mode="fan_out")
         nn.init.constant_(m.bias, 0.0)
 
-    elif classname.find('Conv') != -1:
-        nn.init.kaiming_normal_(m.weight, a=0, mode='fan_in')
+    elif classname.find("Conv") != -1:
+        nn.init.kaiming_normal_(m.weight, a=0, mode="fan_in")
         if m.bias is not None:
             nn.init.constant_(m.bias, 0.0)
-    elif classname.find('BatchNorm') != -1:
+    elif classname.find("BatchNorm") != -1:
         if m.affine:
             nn.init.constant_(m.weight, 1.0)
             nn.init.constant_(m.bias, 0.0)
@@ -26,7 +26,7 @@ def weights_init_kaiming(m):
 
 def weights_init_classifier(m):
     classname = m.__class__.__name__
-    if classname.find('Linear') != -1:
+    if classname.find("Linear") != -1:
         nn.init.normal_(m.weight, std=0.001)
         if m.bias:
             nn.init.constant_(m.bias, 0.0)
@@ -39,10 +39,10 @@ class build_transformer(nn.Module):
         self.cos_layer = cfg.MODEL.COS_LAYER
         self.neck = cfg.MODEL.NECK
         self.neck_feat = cfg.TEST.NECK_FEAT
-        if self.model_name == 'ViT-B-16':
+        if self.model_name == "ViT-B-16":
             self.in_planes = 768
             self.in_planes_proj = 512
-        elif self.model_name == 'RN50':
+        elif self.model_name == "RN50":
             self.in_planes = 2048
             self.in_planes_proj = 1024
         self.num_classes = num_classes
@@ -65,7 +65,12 @@ class build_transformer(nn.Module):
         self.h_resolution = int((cfg.INPUT.SIZE_TRAIN[0] - 16) // cfg.MODEL.STRIDE_SIZE[0] + 1)
         self.w_resolution = int((cfg.INPUT.SIZE_TRAIN[1] - 16) // cfg.MODEL.STRIDE_SIZE[1] + 1)
         self.vision_stride_size = cfg.MODEL.STRIDE_SIZE[0]
-        clip_model = load_clip_to_cpu(self.model_name, self.h_resolution, self.w_resolution, self.vision_stride_size)
+        clip_model = load_clip_to_cpu(
+            self.model_name,
+            self.h_resolution,
+            self.w_resolution,
+            self.vision_stride_size,
+        )
 
         self.image_encoder = clip_model.visual
 
@@ -83,19 +88,23 @@ class build_transformer(nn.Module):
         #     print('camera number is : {}'.format(view_num))
 
     def forward(self, x, label=None, cam_label=None, view_label=None):
-        if self.model_name == 'RN50':
-            image_features_last, image_features, image_features_proj = self.image_encoder(x)  # B,512  B,128,512
+        if self.model_name == "RN50":
+            image_features_last, image_features, image_features_proj = (
+                self.image_encoder(x)
+            )  # B,512  B,128,512
             img_feature_last = nn.functional.avg_pool2d(
-                image_features_last,
-                image_features_last.shape[2:4]).view(x.shape[0], -1)
+                image_features_last, image_features_last.shape[2:4]
+            ).view(x.shape[0], -1)
             img_feature = nn.functional.avg_pool2d(
-                image_features,
-                image_features.shape[2:4]).view(x.shape[0], -1)
+                image_features, image_features.shape[2:4]
+            ).view(x.shape[0], -1)
             img_feature_proj = image_features_proj[0]
 
-        elif self.model_name == 'ViT-B-16':
+        elif self.model_name == "ViT-B-16":
             if cam_label is not None and view_label is not None:
-                cv_embed = self.sie_coe * self.cv_embed[cam_label * self.view_num + view_label]
+                cv_embed = (
+                    self.sie_coe * self.cv_embed[cam_label * self.view_num + view_label]
+                )
             elif cam_label is not None:
                 cv_embed = self.sie_coe * self.cv_embed[cam_label]
             elif view_label is not None:
@@ -103,7 +112,9 @@ class build_transformer(nn.Module):
             else:
                 cv_embed = None
             # B,512  B,128,512
-            image_features_last, image_features, image_features_proj = self.image_encoder(x, cv_embed)
+            image_features_last, image_features, image_features_proj = (
+                self.image_encoder(x, cv_embed)
+            )
             img_feature_last = image_features_last[:, 0]
             img_feature = image_features[:, 0]
             img_feature_proj = image_features_proj[:, 0]
@@ -114,10 +125,14 @@ class build_transformer(nn.Module):
         if self.training:
             cls_score = self.classifier(feat)
             cls_score_proj = self.classifier_proj(feat_proj)
-            return [cls_score, cls_score_proj], [img_feature_last, img_feature, img_feature_proj]
+            return [cls_score, cls_score_proj], [
+                img_feature_last,
+                img_feature,
+                img_feature_proj,
+            ]
 
         else:
-            if self.neck_feat == 'after':
+            if self.neck_feat == "after":
                 # print("Test with feature after BN")
                 return torch.cat([feat, feat_proj], dim=1)
             else:
@@ -126,7 +141,7 @@ class build_transformer(nn.Module):
     def load_param(self, trained_path):
         param_dict = torch.load(trained_path, map_location=torch.device("cpu"))
         for i in self.state_dict():
-            self.state_dict()[i.replace('module.', '')].copy_(param_dict[i])
+            self.state_dict()[i.replace("module.", "")].copy_(param_dict[i])
         # print('Loading pretrained model from {}'.format('/home/mikel.brostrom/yolo_tracking/clip_market1501.pt'))
 
     def load_param_finetune(self, model_path):
@@ -156,6 +171,8 @@ def load_clip_to_cpu(backbone_name, h_resolution, w_resolution, vision_stride_si
     except RuntimeError:
         state_dict = torch.load(model_path, map_location="cpu")
 
-    model = clip.build_model(state_dict or model.state_dict(), h_resolution, w_resolution, vision_stride_size)
+    model = clip.build_model(
+        state_dict or model.state_dict(), h_resolution, w_resolution, vision_stride_size
+    )
 
     return model
