@@ -1,17 +1,19 @@
+# Mikel Broström 🔥 Yolo Tracking 🧾 AGPL-3.0 license
+
 from boxmot.motion.cmc.ecc import ECC
 from boxmot.motion.cmc.orb import ORB
 from boxmot.motion.cmc.sift import SIFT
-from boxmot.motion.cmc.sof import SparseOptFlow
+from boxmot.motion.cmc.sof import SOF
 
 
 def get_cmc_method(cmc_method):
-    if cmc_method == 'ecc':
+    if cmc_method == "ecc":
         return ECC
-    elif cmc_method == 'orb':
+    elif cmc_method == "orb":
         return ORB
-    elif cmc_method == 'sof':
-        return SparseOptFlow
-    elif cmc_method == 'sift':
+    elif cmc_method == "sof":
+        return SOF
+    elif cmc_method == "sift":
         return SIFT
     else:
         return None
