@@ -5,6 +5,7 @@ from boxmot import (
     DeepOcSort,
     OcSort,
     StrongSort,
+    HybridSort,
 )
 
 MOTION_N_APPEARANCE_TRACKING_NAMES = [
@@ -12,10 +13,11 @@ MOTION_N_APPEARANCE_TRACKING_NAMES = [
     "deepocsort",
     "strongsort",
     "boosttrack",
+    "hybridsort",
 ]
 MOTION_ONLY_TRACKING_NAMES = ["ocsort", "bytetrack"]
 
-MOTION_N_APPEARANCE_TRACKING_METHODS = [StrongSort, BotSort, DeepOcSort, BoostTrack]
+MOTION_N_APPEARANCE_TRACKING_METHODS = [StrongSort, BotSort, DeepOcSort, BoostTrack, HybridSort]
 MOTION_ONLY_TRACKING_METHODS = [OcSort, ByteTrack]
 
 ALL_TRACKERS = [
@@ -25,5 +27,6 @@ ALL_TRACKERS = [
     "bytetrack",
     "strongsort",
     "boosttrack",
+    "hybridsort",
 ]
-PER_CLASS_TRACKERS = ["botsort", "deepocsort", "ocsort", "bytetrack"]
+PER_CLASS_TRACKERS = ["botsort", "deepocsort", "ocsort", "bytetrack", "boosttrack", "hybridsort"]
